@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -41,6 +41,11 @@ extern "C" {
 #define CONNMAN_NETWORK_INTERFACE	CONNMAN_SERVICE ".Network"
 #define CONNMAN_CONNECTION_INTERFACE	CONNMAN_SERVICE ".Connection"
 
+extern DBusConnection *connman_dbus_get_connection(void);
+
+extern void connman_dbus_property_append_variant(DBusMessageIter *property,
+					const char *key, int type, void *val);
+
 extern void connman_dbus_dict_append_array(DBusMessageIter *dict,
 				const char *key, int type, void *val, int len);
 extern void connman_dbus_dict_append_variant(DBusMessageIter *dict,
