@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -58,11 +58,12 @@ enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_DEVICE     = 3,
 	CONNMAN_ELEMENT_TYPE_NETWORK    = 4,
 	CONNMAN_ELEMENT_TYPE_SERVICE    = 5,
-	CONNMAN_ELEMENT_TYPE_IPV4       = 6,
-	CONNMAN_ELEMENT_TYPE_IPV6       = 7,
-	CONNMAN_ELEMENT_TYPE_DHCP       = 8,
-	CONNMAN_ELEMENT_TYPE_BOOTP      = 9,
-	CONNMAN_ELEMENT_TYPE_ZEROCONF   = 10,
+	CONNMAN_ELEMENT_TYPE_PPP        = 6,
+	CONNMAN_ELEMENT_TYPE_IPV4       = 7,
+	CONNMAN_ELEMENT_TYPE_IPV6       = 8,
+	CONNMAN_ELEMENT_TYPE_DHCP       = 9,
+	CONNMAN_ELEMENT_TYPE_BOOTP      = 10,
+	CONNMAN_ELEMENT_TYPE_ZEROCONF   = 11,
 
 	CONNMAN_ELEMENT_TYPE_CONNECTION = 42,
 };
@@ -70,11 +71,10 @@ enum connman_element_type {
 enum connman_element_subtype {
 	CONNMAN_ELEMENT_SUBTYPE_UNKNOWN   = 0,
 	CONNMAN_ELEMENT_SUBTYPE_FAKE      = 1,
-	CONNMAN_ELEMENT_SUBTYPE_NETWORK   = 2,
 	CONNMAN_ELEMENT_SUBTYPE_ETHERNET  = 3,
 	CONNMAN_ELEMENT_SUBTYPE_WIFI      = 4,
 	CONNMAN_ELEMENT_SUBTYPE_WIMAX     = 5,
-	CONNMAN_ELEMENT_SUBTYPE_MODEM     = 6,
+	CONNMAN_ELEMENT_SUBTYPE_CELLULAR  = 6,
 	CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH = 7,
 };
 
@@ -90,10 +90,12 @@ struct connman_element {
 	enum connman_element_state state;
 	enum connman_element_policy policy;
 	gboolean enabled;
+	gboolean scanning;
 	gboolean available;
 	gboolean remember;
 	guint16 priority;
 	guint8 strength;
+	gchar *devpath;
 	gchar *devname;
 
 	struct connman_element *parent;
@@ -101,6 +103,14 @@ struct connman_element {
 	struct connman_driver *driver;
 	void *driver_data;
 
+	void (*destruct) (struct connman_element *element);
+
+	union {
+		void *private;
+		struct connman_device *device;
+		struct connman_network *network;
+	};
+
 	GSList *properties;
 
 	struct {
@@ -150,9 +160,8 @@ extern void connman_element_update(struct connman_element *element);
 
 extern int connman_element_set_enabled(struct connman_element *element,
 							gboolean enabled);
-
-extern int connman_element_set_enabled(struct connman_element *element,
-							gboolean enabled);
+extern int connman_element_set_scanning(struct connman_element *element,
+							gboolean scanning);
 
 static inline void *connman_element_get_data(struct connman_element *element)
 {
