@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -21,15 +21,21 @@
 
 #include <glib.h>
 
+#define CONNMAN_API_SUBJECT_TO_CHANGE
+
 #include <connman/dbus.h>
 
 #define NM_SERVICE    "org.freedesktop.NetworkManager"
 #define NM_PATH       "/org/freedesktop/NetworkManager"
 #define NM_INTERFACE  NM_SERVICE
 
+int __connman_dbus_init(DBusConnection *conn);
+void __connman_dbus_cleanup(void);
+
 DBusMessage *__connman_error_failed(DBusMessage *msg);
 DBusMessage *__connman_error_invalid_arguments(DBusMessage *msg);
 DBusMessage *__connman_error_permission_denied(DBusMessage *msg);
+DBusMessage *__connman_error_not_supported(DBusMessage *msg);
 
 int __connman_selftest(void);
 
@@ -71,6 +77,9 @@ enum connman_ipv4_method __connman_ipv4_string2method(const char *method);
 
 #include <connman/resolver.h>
 
+int __connman_resolver_init(void);
+void __connman_resolver_cleanup(void);
+
 int __connman_resolver_selftest(void);
 
 #include <connman/driver.h>
@@ -80,8 +89,16 @@ void __connman_driver_rescan(struct connman_driver *driver);
 #include <connman/element.h>
 
 int __connman_element_init(DBusConnection *conn, const char *device);
+void __connman_element_start(void);
+void __connman_element_stop(void);
 void __connman_element_cleanup(void);
 
+typedef void (* element_cb_t) (struct connman_element *element,
+							gpointer user_data);
+
+void __connman_element_foreach(struct connman_element *element,
+				enum connman_element_type type,
+				element_cb_t callback, gpointer user_data);
 void __connman_element_list(struct connman_element *element,
 					enum connman_element_type type,
 							DBusMessageIter *iter);
@@ -105,11 +122,36 @@ static inline void __connman_element_unlock(struct connman_element *element)
 {
 }
 
+int __connman_detect_init(void);
+void __connman_detect_cleanup(void);
+
+#ifdef HAVE_UDEV
+int __connman_udev_init(void);
+void __connman_udev_cleanup(void);
+#else
+static inline int __connman_udev_init(void)
+{
+	return 0;
+}
+
+static inline void __connman_udev_cleanup(void)
+{
+}
+#endif
+
 #include <connman/device.h>
 
 int __connman_device_init(void);
 void __connman_device_cleanup(void);
 
+#include <connman/network.h>
+
+int __connman_network_init(void);
+void __connman_network_cleanup(void);
+
+void __connman_network_set_device(struct connman_network *network,
+					struct connman_device *device);
+
 #include <connman/rtnl.h>
 
 int __connman_rtnl_init(void);
