@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,6 +38,8 @@ static GNode *element_root = NULL;
 static GSList *driver_list = NULL;
 static gchar *device_filter = NULL;
 
+static gboolean started = FALSE;
+
 static struct {
 	enum connman_property_id id;
 	int type;
@@ -104,6 +106,8 @@ static const char *type2string(enum connman_element_type type)
 		return "network";
 	case CONNMAN_ELEMENT_TYPE_SERVICE:
 		return "service";
+	case CONNMAN_ELEMENT_TYPE_PPP:
+		return "ppp";
 	case CONNMAN_ELEMENT_TYPE_IPV4:
 		return "ipv4";
 	case CONNMAN_ELEMENT_TYPE_IPV6:
@@ -128,16 +132,14 @@ static const char *subtype2string(enum connman_element_subtype type)
 		return "unknown";
 	case CONNMAN_ELEMENT_SUBTYPE_FAKE:
 		return "fake";
-	case CONNMAN_ELEMENT_SUBTYPE_NETWORK:
-		return "network";
 	case CONNMAN_ELEMENT_SUBTYPE_ETHERNET:
 		return "ethernet";
 	case CONNMAN_ELEMENT_SUBTYPE_WIFI:
 		return "wifi";
 	case CONNMAN_ELEMENT_SUBTYPE_WIMAX:
 		return "wimax";
-	case CONNMAN_ELEMENT_SUBTYPE_MODEM:
-		return "modem";
+	case CONNMAN_ELEMENT_SUBTYPE_CELLULAR:
+		return "cellular";
 	case CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH:
 		return "bluetooth";
 	}
@@ -145,28 +147,6 @@ static const char *subtype2string(enum connman_element_subtype type)
 	return NULL;
 }
 
-static const char *subtype2description(enum connman_element_subtype type)
-{
-	switch (type) {
-	case CONNMAN_ELEMENT_SUBTYPE_UNKNOWN:
-	case CONNMAN_ELEMENT_SUBTYPE_FAKE:
-	case CONNMAN_ELEMENT_SUBTYPE_NETWORK:
-		return NULL;
-	case CONNMAN_ELEMENT_SUBTYPE_ETHERNET:
-		return "Ethernet";
-	case CONNMAN_ELEMENT_SUBTYPE_WIFI:
-		return "Wireless";
-	case CONNMAN_ELEMENT_SUBTYPE_WIMAX:
-		return "WiMAX";
-	case CONNMAN_ELEMENT_SUBTYPE_MODEM:
-		return "Modem";
-	case CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH:
-		return "Bluetooth";
-	}
-
-	return NULL;
-}
-
 const char *__connman_element_policy2string(enum connman_element_policy policy)
 {
 	switch (policy) {
@@ -411,26 +391,6 @@ static void emit_scanning_signal(DBusConnection *conn,
 	g_dbus_send_message(conn, signal);
 }
 
-static DBusMessage *do_update(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-
-	DBG("conn %p", conn);
-
-	if (element->enabled == FALSE)
-		return __connman_error_failed(msg);
-
-	if (element->driver && element->driver->update) {
-		DBG("Calling update callback");
-		if (element->driver->update(element) < 0)
-			return __connman_error_failed(msg);
-
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
 static DBusMessage *do_enable(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -477,214 +437,12 @@ static DBusMessage *do_disable(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static void append_networks(struct connman_element *element,
-						DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Networks";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
-	__connman_element_list(element, CONNMAN_ELEMENT_TYPE_NETWORK, &iter);
-
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static DBusMessage *device_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict, entry;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	str = subtype2description(element->subtype);
-	if (str != NULL && element->devname != NULL) {
-		char *name = g_strdup_printf("%s (%s)", str, element->devname);
-		if (name != NULL)
-			connman_dbus_dict_append_variant(&dict, "Name",
-						DBUS_TYPE_STRING, &name);
-		g_free(name);
-	}
-
-	str = subtype2string(element->subtype);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Type",
-						DBUS_TYPE_STRING, &str);
-
-	str = __connman_element_policy2string(element->policy);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Policy",
-						DBUS_TYPE_STRING, &str);
-
-	connman_dbus_dict_append_variant(&dict, "Powered",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH) {
-		connman_dbus_dict_append_variant(&dict, "Scanning",
-					DBUS_TYPE_BOOLEAN, &element->scanning);
-
-		dbus_message_iter_open_container(&dict, DBUS_TYPE_DICT_ENTRY,
-								NULL, &entry);
-		append_networks(element, &entry);
-		dbus_message_iter_close_container(&dict, &entry);
-	}
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *device_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter, value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Powered") == TRUE) {
-		dbus_bool_t powered;
-
-		dbus_message_iter_get_basic(&value, &powered);
-
-		if (powered == TRUE)
-			do_enable(conn, msg, element);
-		else
-			do_disable(conn, msg, element);
-	} else
-		set_common_property(element, name, &value);
-
-	__connman_element_store(element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static int parse_network_dict(DBusMessageIter *iter, const char **ssid,
-				const char **security, const char **passphrase)
-{
-	while (dbus_message_iter_get_arg_type(iter) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key;
-
-		dbus_message_iter_recurse(iter, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		switch (dbus_message_iter_get_arg_type(&value)) {
-		case DBUS_TYPE_STRING:
-			if (g_str_equal(key, "WiFi.SSID") == TRUE)
-				dbus_message_iter_get_basic(&value, ssid);
-			else if (g_str_equal(key, "WiFi.Security") == TRUE)
-				dbus_message_iter_get_basic(&value, security);
-			else if (g_str_equal(key, "WiFi.Passphrase") == TRUE)
-				dbus_message_iter_get_basic(&value, passphrase);
-			break;
-		}
-
-		dbus_message_iter_next(iter);
-	}
-
-	return 0;
-}
-
-static DBusMessage *device_create_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	struct connman_element *network;
-	DBusMessageIter iter, array;
-	const char *ssid = NULL, *security = NULL, *passphrase = NULL;
-
-	DBG("conn %p", conn);
-
-	if (element->subtype != CONNMAN_ELEMENT_SUBTYPE_WIFI)
-		return __connman_error_invalid_arguments(msg);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_recurse(&iter, &array);
-	parse_network_dict(&array, &ssid, &security, &passphrase);
-	if (ssid == NULL)
-		return __connman_error_invalid_arguments(msg);
-
-	DBG("ssid %s security %s passphrase %s", ssid, security, passphrase);
-
-	network = connman_element_create(ssid);
-
-	network->type = CONNMAN_ELEMENT_TYPE_NETWORK;
-	network->index = element->index;
-
-	network->remember = TRUE;
-
-	connman_element_add_static_property(network, "Name",
-						DBUS_TYPE_STRING, &ssid);
-
-	connman_element_add_static_array_property(element, "WiFi.SSID",
-					DBUS_TYPE_BYTE, &ssid, strlen(ssid));
-
-	network->wifi.security = g_strdup(security);
-	network->wifi.passphrase = g_strdup(passphrase);
-
-	connman_element_register(network, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_OBJECT_PATH, &network->path,
-							DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *device_remove_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
 static DBusMessage *network_get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct connman_element *element = data;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
-	const char *str;
 
 	DBG("conn %p", conn);
 
@@ -703,11 +461,6 @@ static DBusMessage *network_get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_variant(&dict, "Device",
 				DBUS_TYPE_OBJECT_PATH, &element->parent->path);
 
-	str = __connman_element_policy2string(element->policy);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Policy",
-						DBUS_TYPE_STRING, &str);
-
 	connman_dbus_dict_append_variant(&dict, "Available",
 					DBUS_TYPE_BOOLEAN, &element->available);
 
@@ -786,6 +539,10 @@ static DBusMessage *connection_get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_variant(&dict, "Type",
 						DBUS_TYPE_STRING, &str);
 
+	if (element->devname != NULL)
+		connman_dbus_dict_append_variant(&dict, "Interface",
+					DBUS_TYPE_STRING, &element->devname);
+
 	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
 			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX)
 		connman_dbus_dict_append_variant(&dict, "Strength",
@@ -834,15 +591,6 @@ static DBusMessage *connection_set_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static GDBusMethodTable device_methods[] = {
-	{ "GetProperties", "",      "a{sv}", device_get_properties },
-	{ "SetProperty",   "sv",    "",      device_set_property   },
-	{ "CreateNetwork", "a{sv}", "o",     device_create_network },
-	{ "RemoveNetwork", "o",     "",      device_remove_network },
-	{ "ProposeScan",   "",      "",      do_update             },
-	{ },
-};
-
 static GDBusMethodTable network_methods[] = {
 	{ "GetProperties", "",   "a{sv}", network_get_properties },
 	{ "SetProperty",   "sv", "",      network_set_property   },
@@ -862,6 +610,53 @@ static GDBusSignalTable element_signals[] = {
 	{ },
 };
 
+struct foreach_data {
+	enum connman_element_type type;
+	element_cb_t callback;
+	gpointer user_data;
+};
+
+static gboolean foreach_callback(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct foreach_data *data = user_data;
+
+	DBG("element %p name %s", element, element->name);
+
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return FALSE;
+
+	if (data->type != CONNMAN_ELEMENT_TYPE_UNKNOWN &&
+					data->type != element->type)
+		return FALSE;
+
+	if (data->callback)
+		data->callback(element, data->user_data);
+
+	return FALSE;
+}
+
+void __connman_element_foreach(struct connman_element *element,
+				enum connman_element_type type,
+				element_cb_t callback, gpointer user_data)
+{
+	struct foreach_data data = { type, callback, user_data };
+	GNode *node;
+
+	DBG("");
+
+	if (element != NULL) {
+		node = g_node_find(element_root, G_PRE_ORDER,
+						G_TRAVERSE_ALL, element);
+		if (node == NULL)
+			return;
+	} else
+		node = element_root;
+
+	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+						foreach_callback, &data);
+}
+
 struct append_filter {
 	enum connman_element_type type;
 	DBusMessageIter *iter;
@@ -881,10 +676,6 @@ static gboolean append_path(GNode *node, gpointer user_data)
 					filter->type != element->type)
 		return FALSE;
 
-	if (filter->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_NETWORK)
-		return FALSE;
-
 	dbus_message_iter_append_basic(filter->iter,
 				DBUS_TYPE_OBJECT_PATH, &element->path);
 
@@ -1053,6 +844,9 @@ int connman_driver_register(struct connman_driver *driver)
 	driver_list = g_slist_insert_sorted(driver_list, driver,
 							compare_priority);
 
+	if (started == FALSE)
+		return 0;
+
 	if (element_root != NULL)
 		g_node_traverse(element_root, G_PRE_ORDER,
 				G_TRAVERSE_ALL, -1, probe_driver, driver);
@@ -1187,6 +981,8 @@ void connman_element_unref(struct connman_element *element)
 				g_atomic_int_get(&element->refcount) - 1);
 
 	if (g_atomic_int_dec_and_test(&element->refcount) == TRUE) {
+		if (element->destruct)
+			element->destruct(element);
 		free_properties(element);
 		g_free(element->ipv4.address);
 		g_free(element->ipv4.netmask);
@@ -1195,6 +991,7 @@ void connman_element_unref(struct connman_element *element)
 		g_free(element->ipv4.broadcast);
 		g_free(element->ipv4.nameserver);
 		g_free(element->devname);
+		g_free(element->devpath);
 		g_free(element->path);
 		g_free(element->name);
 		g_free(element);
@@ -1748,10 +1545,11 @@ gboolean connman_element_match_static_property(struct connman_element *element,
 	return result;
 }
 
-static void append_devices(DBusMessageIter *entry)
+static void append_networks(struct connman_element *element,
+						DBusMessageIter *entry)
 {
 	DBusMessageIter value, iter;
-	const char *key = "Devices";
+	const char *key = "Networks";
 
 	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
 
@@ -1761,31 +1559,12 @@ static void append_devices(DBusMessageIter *entry)
 
 	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
 				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, &iter);
+	__connman_element_list(element, CONNMAN_ELEMENT_TYPE_NETWORK, &iter);
 	dbus_message_iter_close_container(&value, &iter);
 
 	dbus_message_iter_close_container(entry, &value);
 }
 
-static void emit_devices_signal(DBusConnection *conn)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_devices(&entry);
-
-	g_dbus_send_message(conn, signal);
-}
-
 static void emit_networks_signal(DBusConnection *conn,
 					struct connman_element *device)
 {
@@ -1893,11 +1672,35 @@ static void set_signal_strength(struct connman_element *connection)
 	}
 }
 
+static void probe_element(struct connman_element *element)
+{
+	GSList *list;
+
+	DBG("element %p name %s", element, element->name);
+
+	for (list = driver_list; list; list = list->next) {
+		struct connman_driver *driver = list->data;
+
+		if (match_driver(element, driver) == FALSE)
+			continue;
+
+		DBG("driver %p name %s", driver, driver->name);
+
+		if (driver->probe(element) == 0) {
+			__connman_element_lock(element);
+			element->driver = driver;
+			__connman_element_unlock(element);
+
+			enable_element(element);
+			break;
+		}
+	}
+}
+
 static void register_element(gpointer data, gpointer user_data)
 {
 	struct connman_element *element = data;
 	const gchar *basepath;
-	GSList *list;
 	GNode *node;
 
 	__connman_element_lock(element);
@@ -1928,18 +1731,6 @@ static void register_element(gpointer data, gpointer user_data)
 
 	g_node_append_data(node, element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_DEVICE_INTERFACE,
-					device_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s device",
-								element->path);
-		else
-			emit_devices_signal(connection);
-	}
-
 	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
 		if (g_dbus_register_interface(connection, element->path,
 					CONNMAN_NETWORK_INTERFACE,
@@ -1967,23 +1758,10 @@ static void register_element(gpointer data, gpointer user_data)
 
 	__connman_element_store(element);
 
-	for (list = driver_list; list; list = list->next) {
-		struct connman_driver *driver = list->data;
+	if (started == FALSE)
+		return;
 
-		if (match_driver(element, driver) == FALSE)
-			continue;
-
-		DBG("driver %p name %s", driver, driver->name);
-
-		if (driver->probe(element) == 0) {
-			__connman_element_lock(element);
-			element->driver = driver;
-			__connman_element_unlock(element);
-
-			enable_element(element);
-			break;
-		}
-	}
+	probe_element(element);
 }
 
 /**
@@ -2004,8 +1782,7 @@ int connman_element_register(struct connman_element *element,
 	if (element->devname == NULL)
 		element->devname = g_strdup(element->name);
 
-	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
+	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE) {
 		if (g_pattern_match_simple(device_filter,
 						element->devname) == FALSE) {
 			DBG("ignoring %s [%s] device", element->name,
@@ -2046,6 +1823,9 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 	if (element == root)
 		return FALSE;
 
+	if (node != NULL)
+		g_node_unlink(node);
+
 	if (element->driver) {
 		disable_element(element);
 
@@ -2057,10 +1837,8 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		__connman_element_unlock(element);
 	}
 
-	if (node != NULL) {
-		g_node_unlink(node);
+	if (node != NULL)
 		g_node_destroy(node);
-	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
 		if (__connman_element_count(NULL,
@@ -2079,14 +1857,6 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 						CONNMAN_NETWORK_INTERFACE);
 	}
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
-		emit_devices_signal(connection);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_DEVICE_INTERFACE);
-	}
-
 	connman_element_unref(element);
 
 	return FALSE;
@@ -2201,10 +1971,47 @@ int __connman_element_init(DBusConnection *conn, const char *device)
 	element_root = g_node_new(element);
 
 	__connman_device_init();
+	__connman_network_init();
 
 	return 0;
 }
 
+static gboolean probe_node(GNode *node, gpointer data)
+{
+	struct connman_element *element = node->data;
+
+	DBG("element %p name %s", element, element->name);
+
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return FALSE;
+
+	if (element->driver)
+		return FALSE;
+
+	probe_element(element);
+
+	return FALSE;
+}
+
+void __connman_element_start(void)
+{
+	DBG("");
+
+	g_node_traverse(element_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+							probe_node, NULL);
+
+	started = TRUE;
+
+	__connman_detect_init();
+}
+
+void __connman_element_stop(void)
+{
+	DBG("");
+
+	__connman_detect_cleanup();
+}
+
 static gboolean free_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -2241,6 +2048,7 @@ void __connman_element_cleanup(void)
 {
 	DBG("");
 
+	__connman_network_cleanup();
 	__connman_device_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
